; (function () {
    if (typeof ClientRect != 'undefined') {
        ClientRect.prototype.pcs_toObject = function () {
            return { x: this.left, y: this.top, width: this.width, height: this.height };
        }
    }

    if (typeof DOMRect != 'undefined') {
        DOMRect.prototype.pcs_toObject = function () {
            return { x: this.x, y: this.y, width: this.width, height: this.height };
        }
    }

    Element.prototype.pcs_getContentHeight = function () {
        return Math.max(this.scrollHeight, this.clientHeight);
    }

    Element.prototype.pcs_getContentWidth = function () {
        return Math.max(this.scrollWidth, this.clientWidth);
    }

    Element.prototype.pcs_getContentSize = function () {
        return { width: this.pcs_getContentWidth(), height: this.pcs_getContentHeight() };
    }

    Element.prototype.pcs_getContentOffset = function () {
        return { x: this.scrollLeft, y: this.scrollTop };
    }

    Element.prototype.pcs_getRegion = function () {
        return { frame: this.getBoundingClientRect().pcs_toObject(), contentSize: this.pcs_getContentSize() };
    }

    Element.prototype.pcs_getScrollStatus = function () {
        return { contentOffset: this.pcs_getContentOffset(), region: this.pcs_getRegion() };
    }

    Element.prototype.pcs_getFrameInViewPort = function () {
        let rect = this.getBoundingClientRect();

        let window = this.ownerDocument.defaultView;
        if (window != null) {
            rect.x += window.pageXOffset;
            rect.y += window.pageYOffset;
        }

        return rect.pcs_toObject();
    }

    window.pcs_getHighestNode = function () {
        var highestNode = undefined;
        var maxHeight = 0;

        function findHighestNode(nodesList) {
            for (let i = 0; i < nodesList.length; i++) {
                let node = nodesList[i];

                if (node instanceof Element) {
                    let height = node.pcs_getContentHeight();

                    if (height > maxHeight) {
                        maxHeight = height;
                        highestNode = node;
                    }

                    if (node.childNodes.length > 0) {
                        findHighestNode(node.childNodes);
                    }
                }
            }
        }

        findHighestNode(this.document.documentElement.childNodes);

        return highestNode;
    }

    window.pcs_getScrollNode = function () {
        return this.pcs_getHighestNode();
    }

    window.addEventListener("message", function (event) {
        let funcPath = event.data.func;
        if (funcPath != undefined) {
            // let func = funcPath.split('.').reduce((previous, current) => previous[current], this);
            let func = this[funcPath];
            if (func instanceof Function) {
                func.apply(this, event.data.args);
                return
            }
        }
    });

    window.pcs_dynamicCall = function (func, windowIndex) {
        if (this[func] instanceof Function) {
            if (windowIndex == 0) {
                return this[func].apply(this, Array.prototype.slice.call(arguments, 1));
            } else {
                return this.frames[windowIndex - 1].postMessage({ func: func, args: Array.prototype.slice.call(arguments, 1) }, "*");
            }
        }
    }

    window.pcs_scroll = function (windowIndex, point) {
        this.pcs_getScrollNode().scroll(point.x, point.y);
        let result = { windowIndex: windowIndex };
        this.webkit.messageHandlers.scroll.postMessage(result);
        return result;
    }

    window.pcs_getScrollStatus = function (windowIndex) {
        let status = this.pcs_getScrollNode().pcs_getScrollStatus();
        status.windowIndex = windowIndex;
        this.webkit.messageHandlers.getScrollStatus.postMessage(status);
        return status;
    }

    window.pcs_getIFrameInfos = function () {
        let srcs = [];
        let frames = this.document.getElementsByTagName("iframe");
        for (let i = 0; i < frames.length; i++) {
            let frame = frames[i];
            if (frame.src.startsWith('http')) {
                let rect = frame.getBoundingClientRect();
                if (rect.width > 0 && rect.height > 0) {
                    let info = { index: i, rect: rect.pcs_toObject(), src: frame.src };
                    srcs.push(info);
                }
            }
        }
        return srcs;
    }
})();
